/**
 * @file registers the toolbar button and binds functionality to it.
 */

import { Plugin } from 'ckeditor5/src/core';
import { ButtonView } from 'ckeditor5/src/ui';
import icon from '../../../../icons/ephotoDamIcon.svg';

var url;

export default class EphotoDamUI extends Plugin {

  /**
   * create the Ephoto Dam button
   */
  init() {

    (function ($, Drupal, drupalSettings) {
      Drupal.behaviors.MyModuleBehavior = {
        attach: function (context, settings) {
          // get color_body value with "drupalSettings.mymodule.color_body"
          url = drupalSettings.ephoto_dam;
        }
      };
    })(jQuery, Drupal, drupalSettings);

    const editor = this.editor;

    // Allow <iframe> elements in the model.
    editor.model.schema.register( 'iframe', {
      allowWhere: '$text',
      allowContentOf: '$block'
    } );
    // Allow <iframe> elements in the model to have all attributes.
    editor.model.schema.addAttributeCheck( context => {
      if ( context.endsWith( 'iframe' ) ) {
        return true;
      }
    } );
    // View-to-model converter converting a view <iframe> with all its attributes to the model.
    editor.conversion.for( 'upcast' ).elementToElement( {
      view: 'iframe',
      model: ( viewElement, modelWriter ) => {
        return modelWriter.writer.createElement( 'iframe', viewElement.getAttributes() );
      }
    } );

    // Model-to-view converter for the <iframe> element (attributes are converted separately).
    editor.conversion.for( 'downcast' ).elementToElement( {
      model: 'iframe',
      view: 'iframe'
    } );

    // Model-to-view converter for <iframe> attributes.
    // Note that a lower-level, event-based API is used here.
    editor.conversion.for( 'downcast' ).add( dispatcher => {
      dispatcher.on( 'attribute', ( evt, data, conversionApi ) => {
        // Convert <iframe> attributes only.

        if(data.item.name == 'iframe'){
          const viewWriter = conversionApi.writer;
          const viewIframe = conversionApi.mapper.toViewElement( data.item );

          // In the model-to-view conversion we convert changes.
          // An attribute can be added or removed or changed.
          // The below code handles all 3 cases.
          if ( data.attributeNewValue ) {
            viewWriter.setAttribute( data.attributeKey, data.attributeNewValue, viewIframe );
          } else {
            viewWriter.removeAttribute( data.attributeKey, viewIframe );
          }
        }

        if ( data.attributeNewValue == null || data.attributeNewValue.classes == null || !data.attributeNewValue.classes.includes("ckeditor_ephoto_dam") ) {
          return;
        }

        const id = data.attributeNewValue.attributes.id;
        window.requestAnimationFrame(() => {

          this.addImageZoomConfig(id);

          document.addEventListener("keyup",(e) => {
            var auth_Id = this.getCookie("ephoto_dam_authid");
            if(auth_Id && e.target.parentElement && e.target.parentElement.parentElement && e.target.parentElement.parentElement.id == id){
              var currentLegend = e.target.innerText;
              var id_media = this.getDataCalc(id,"id_image")
              this.replaceCaptionMedia(id,id_media,auth_Id,currentLegend);
            }
          });

          let mediaCalc = document.getElementById("calc_" + id);
          if(mediaCalc){

            let model = this.editor.model;

            var x = new MutationObserver(function (e) {

              if (e[0].removedNodes.length > 0){

                let root = model.document.getRoot();
                let children = root.getChildren();
                let nextSelect;
                let childToDelete;

                for(let child of children){
                  if(child.name == 'htmlDiv' && child.getAttribute("htmlDivAttributes") && child.getAttribute("htmlDivAttributes").classes && child.getAttribute("htmlDivAttributes").classes.includes("ckeditor_ephoto_dam") && child.getAttribute("htmlDivAttributes").attributes.id == id && document.getElementById(id).querySelector("img") == null){
                    childToDelete = child;

                    model.change( writer => {
                      var selection = childToDelete;
                      //writer.setSelection(selection, 'on' );
                      writer.remove(selection);
                    } );

                    let index = child.index;
                    if(root.getChild(index + 1) != null){
                      nextSelect = root.getChild(index + 1);
                    }
                    else{
                      if(root.getChild(index - 1) != null){
                        nextSelect = root.getChild(index - 1);
                      }
                      else{
                        nextSelect = root.getChild(0);
                      }
                    }

                    break;
                  }
               }

                model.change( writer => {
                  writer.setSelection(nextSelect, 'on' );
                } );

              }
            });

            x.observe(document.getElementById(id), { childList: true });

          }

      });

        const viewWriter = conversionApi.writer;
        const viewIframe = conversionApi.mapper.toViewElement( data.item );

        // In the model-to-view conversion we convert changes.
        // An attribute can be added or removed or changed.
        // The below code handles all 3 cases.
        if ( data.attributeNewValue ) {
          viewWriter.setAttribute( data.attributeKey, data.attributeNewValue, viewIframe );
        } else {
          viewWriter.removeAttribute( data.attributeKey, viewIframe );
        }
      } );
    } );

    // This will register the toolbar button.
    editor.ui.componentFactory.add('simpleBox', (locale) => {
      const command = editor.commands.get('insertEphotoDam');
      const buttonView = new ButtonView(locale);

      // Create the toolbar button.
      buttonView.set({
        label: editor.t('EphotoDam'),
        icon,
        tooltip: false,
        class : "ephoto_dam_tooltip_icon"
      });

      // Bind the state of the button to the command.
      buttonView.bind('isOn', 'isEnabled').to(command, 'value', 'isEnabled');

      // Execute the command when the button is clicked (executed).
      this.listenTo(buttonView, 'execute', () =>
        editor.execute('insertEphotoDam'),
      );

      return buttonView;
    });
  }

  replaceCaptionMedia(id,id_media,auth_Id,currentLegend){

    const matches = currentLegend.match(/\[(.*?)\]/g);

    if (matches) {

      var urlInfo = drupalSettings.ephoto_dam.server_url + 'api/' + auth_Id + '/file/getInfo?id=' + id_media;
      var conn = new XMLHttpRequest();
      conn.open("GET", urlInfo);
      conn.send();
      var parser = new DOMParser();
      var self = this;

      conn.onreadystatechange = function () {
        if (conn.readyState === 4) {
          if(conn.status === 200) {
            var xmlDoc = parser.parseFromString(conn.responseText,"text/xml");
            var fields = xmlDoc.getElementById("Desc");

            matches.forEach(match => {

              var newValue = null;

              fields.querySelectorAll('field').forEach(element => {
                var fieldName = element.querySelector('name');

                var matchText = match.slice(1,match.length - 1);
                if(fieldName.textContent && fieldName.textContent == matchText ){
                  newValue = element.querySelector('value').textContent;
                  return;
                }

              });

              if(newValue){
                const newLegend = document.getElementById(id).querySelector("figcaption").innerHTML.replaceAll(match,newValue);

                self.updateLegend(id,newLegend);
              }
            });

          }
        }
      };

    }
  }

  addImageZoomConfig(id){
    if(this.getDataCalc(id,"ephoto_dam") == "image"){

      var noContext = document.getElementById("media_" + id);

      noContext.addEventListener("contextmenu", (e) => {
        e.preventDefault();

        let menuCustomToDelete = document.getElementsByClassName("ephoto_dam_properties");
        while(menuCustomToDelete.length > 0){
          menuCustomToDelete[0].parentNode.removeChild(menuCustomToDelete[0]);
        }

        var data = document.getElementById("calc_" + id).getAttribute("data");
        var value = data.split("zoom:{")[1].split("}")[0];

        var contextMenu = '<div id="context-menu">';
        if(value == "true"){
          contextMenu += '<div class="item">' + Drupal.t('Disable zoom') + '</div>';
        }else{
          contextMenu += '<div class="item">' + Drupal.t('Enable zoom') + '</div>';
        }
        contextMenu += '</div>';

        var divAbove = document.createElement("div");
        divAbove.innerHTML = contextMenu;
        divAbove.id = "ephoto_dam_properties_" + id;
        divAbove.className = "ephoto_dam_properties";

        document.getElementsByTagName("body")[0].appendChild(divAbove);
        let menuCustom = document.getElementById("ephoto_dam_properties_" + id);
        let x = e.clientX, y = e.clientY;

        menuCustom.style.left = `${x}px`;
        menuCustom.style.top = `${y}px`;
        menuCustom.style.visibility = "visible";
        menuCustom.style.position = "fixed";
        menuCustom.style.zIndex = "2";

        menuCustom.addEventListener("click",(e) => {
          if(value == "true"){
            this.setDataZoom(id,"false");
          }else{
            this.setDataZoom(id,"true");
            this.editor.updateSourceElement();
          }
        });
      });

      document.addEventListener("wheel",(e) => {
        let menuCustom = document.getElementById("ephoto_dam_properties_" + id);
        if(menuCustom){
          menuCustom.remove();
        }
      });

      document.addEventListener("click",(e) => {
        let menuCustom = document.getElementById("ephoto_dam_properties_" + id);
        if(menuCustom){
          menuCustom.remove();
        }
      });
    }
  }

  updateLegend(id,legend){
    var doc = this.dataToHtml();
    doc.getElementById("media_" + id).setAttribute("data-caption",legend);
    var retour = doc.querySelector("body");
    this.editor.setData(retour.innerHTML);
  }

  setDataZoom(id, newData){
    var doc = this.dataToHtml();

    var data = doc.getElementById("calc_" + id).getAttribute("data");

    var tempdata = data.split("zoom:{")[0];
    tempdata += "zoom:{" + newData + "}";
    tempdata += data.split("zoom:{")[1].split("}")[1];

    doc.getElementById("calc_" + id).setAttribute("data",tempdata);

    var retour = doc.querySelector("body");

    this.editor.setData(retour.innerHTML);
  }


  dataToHtml(){
    var tempdata = this.editor.getData();
    var parser = new DOMParser();
    var doc = parser.parseFromString(tempdata, 'text/html');
    return doc;
  }

  getDataCalc(id, dataToRetrieve){
    var data = document.getElementById("calc_" + id).getAttribute("data");
    var value = data.split(dataToRetrieve + ":{")[1].split("}")[0];

    return value;
  }

  getCookie(name) {
    let cookie = {};
    document.cookie.split(';').forEach(function (el) {
      let split = el.split('=');
      cookie[split[0].trim()] = split.slice(1).join("=");
    })
    return cookie[name];
  }

}
