/**
 * @file defines InsertEphotoDamCommand, which is executed when the
 * toolbar button is pressed.
 */

var script = document.createElement('script');
var url = drupalSettings.ephoto_dam.server_url;
script.type = 'text/javascript';
script.src = url + 'api/apiJS.js';
document.body.appendChild(script);

import { Command } from 'ckeditor5/src/core';


export default class InsertEphotoDamCommand extends Command {


  /**
   * This function calls the Ephoto Dam API
   *
   * It starts by pulling the API from the entered Ephoto Dam URL
   * It sets the buttons sizes which will be used
   * Finally it opens the Ephoto Dam into a popup window
   */
  execute() {

    const { model } = this.editor;

    var urlAuth = drupalSettings.ephoto_dam.server_url + 'api/auth/getNewAuthID';
    var conn = new XMLHttpRequest();
    conn.open("GET", urlAuth);
    conn.send();

    var self = this;
    conn.onreadystatechange = function () {
        if (conn.readyState === 4) {
            if(conn.status === 200) {
              var expiration = Date.now() + (60 * 30);
              document.cookie = "ephoto_dam_authid=" + conn.responseText + "; expires=" + expiration;
              drupalSettings.ephoto_dam.auth_id = conn.responseText;
              self.connectEphoto();
            }
        }
    };
  }

  connectEphoto(){
    var ephoto = new EphotoDam({
      server: drupalSettings.ephoto_dam.server_url,
      authID: drupalSettings.ephoto_dam.auth_id,
      client: 'bD3ui8br'
    });

    ephoto.connect();
    ephoto.File.setMode('embed');
    const config = this.editor.config.get('ephoto_dam');

    ephoto.File.setButtons(ephoto.IMAGE_FILES, [{
      'definition': !config.zoom && config.images_size <= 320 ? 'low' : 'middle',
      'size': config.images_size
    }]);

    ephoto.File.setButtons(ephoto.MOVIE_FILES, [{
      'definition': 'custom',
      'size': config.videos_size
    }]);

    ephoto.File.setButtons(ephoto.DOCUMENT_FILES, [{
      'definition': 'custom',
      'size': config.documents_size
    }]);

    ephoto.File.setButtons(ephoto.MODEL_FILES,[{
      'definition': 'custom',
      'size': config.documents_size
    }]);

    ephoto.File.callOnFileReceived(this.insertFile.bind(this));

    ephoto.File.enableDCore();
    ephoto.File.get();
  }

  /**
   * Retrieve the imported media from the returned objects and add it to the page
   *
   * @param {string} html
   * @param {ephotoDamObject} dcore
   */
  insertFile(html, dcore) {

    const { model } = this.editor;
    const config = this.editor.config.get('ephoto_dam');
    const editor = this.editor;
    const id = "ephoto_dam_" + this.genRandonString(10);

    let tag;
    if (!html || html === 'not logged in' || !dcore) {
        return;
    }

    if (html === 'failure') { return; }
    if (html === 'fileDoesNotExist') {  return; }

    let dctype = this.getValueOfXmlNode(dcore, 'dc:type');
    let dcid = this.getValueOfXmlNode(dcore, 'dc:identifier');

    switch(dctype) {
      case 'Image':
        let embed = this.getURLFromString(html, 'embed');
        let self = this;

        fetch(embed)
        .then(function (response) {
            switch (response.status) {
                // status "OK"
                case 200:
                    return response.text();

                // status "Not Found"
                case 404:
                    throw response;
            }
        })
        .then(function (template) {
          //const myArr = JSON.parse(template);

          let splitted = template.split('<img src="')[1];
          let urlImage = splitted.split('"')[0];
          var url = new URL(urlImage);
          var width = url.searchParams.get("width");

          let imageSrc = '<img id="media_' + id + '" class="unselectable img_ephoto_dam" src="' + urlImage + '" alt="' + config.captions_format + '"/>';

          let data = 'ephoto_dam:{image}; align:{' + config.align + '}; src:{' + urlImage + '}; alt:{' + config.captions_format + '}; zoom:{' + config.zoom + '}; captions:{' + config.captions + '}; width:{' + width + '}; id_image:{' + dcid + '} captions_format:{' + config.captions_format + '};';

          self.insertImageModel(imageSrc,id,config,data,dctype,dcid);

        })
        .catch(function (response) {
            // "Not Found"
        });
        break;

        default:
          var htmlObject = document.createElement('div');
          htmlObject.innerHTML = html;

          let src = htmlObject.children[0].src;
          var url = new URL(src);
          let width = url.searchParams.get("width");
          let height = url.searchParams.get("height");
          let title = htmlObject.children[0].title;

          let dcThumbnail = this.getValueOfXmlNode(dcore, 'dc:relation').replace("/small/","/medium/");

          tag = '<iframe id=\'ephoto_dam_iframe_' + id + '\' class=\'ephoto_dam_iframe\'' + html.split("<iframe")[1].replaceAll("\"","'");

          let imageSrc = '<img id="media_' + id + '" class="unselectable" src="' + dcThumbnail + '" alt="' + config.captions_format + /*'" style ="height:' + height + 'px;' width="'+width+'px*/ '"/>';
          let data = 'ephoto_dam:{' + dctype + '}; align:{' + config.align + '}; alt:{' + config.captions_format + '}; zoom:{' + config.zoom + '}; captions:{' + config.captions + '}; height:{' + height + '}; width:{' + width + '}; id_image:{' + dcid + '} tag:{' + tag + '}; embed:{' + src + '}; title:{' + title + '}; captions_format:{' + config.captions_format + '};';

          this.insertImageModel(imageSrc,id,config,data,dctype,dcid);

          break;
        }
    }

      /**
   * Make the image model for the media to be inserted in the edit version
   *
   * @param {string} mediaBalise balise of the media
   * @param {string} id id of the media on the page
   * @param {array} config configuration of the application
   * @param {string} data div info to be used in the display version
   * @param {string} type  type of the media
   * @param {string} id_media id of the media in Ephoto Dam
   */
    insertImageModel(mediaBalise,id,config,data,type,id_media){
      this.mediaHandler('<div class="ckeditor_ephoto_dam ephoto_dam_' + config.align + ' unselectable ' + type + '_ephoto_dam" id="' + id + '" contenteditable="false"> <div id="calc_' + id + '" class="calc" data="' + data + '"></div>' + mediaBalise + '</div>',id,config,id_media);
    }

    /**
   * generate a random string
   *
   * @param length int
   * @returns sttring
   */
    genRandonString(length) {
      const chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
      const charLength = chars.length;
      let result = '';
      for ( let i = 0; i < length; i++ ) {
         result += chars.charAt(Math.floor(Math.random() * charLength));
      }
      return result;
   }

  /**
   * Add the media to the page
   *
   * @param {string} imageSrc
   * @param {string} id id of the media on the page
   * @param {array} config configuration of the application
   * @param {string} $id_media id of the media in Ephoto Dam
   */
  mediaHandler(imageSrc,id,config,id_media){

    const viewFragmentImage = this.editor.data.processor.toView(imageSrc);

    const modelFragmentImage = this.editor.data.toModel(viewFragmentImage);

    this.editor.model.insertContent( modelFragmentImage );

    if(config.captions){
      this.replaceCaptionMedia(id,id_media,drupalSettings.ephoto_dam.auth_id,config.captions_format);
    }
  }

    /**
   * Replace the caption on the media
   * @param {string} id id of the media on the page
   * @param {string} id_media id of the media in Ephoto Dam
   * @param {int} auth_Id id of the user in Ephoto Dam
   * @param {string} currentLegend the caption currently displayed
   */
  replaceCaptionMedia(id,id_media,auth_Id,currentLegend){

    const matches = currentLegend.match(/\[(.*?)\]/g);

    if (matches && auth_Id) {

      var urlInfo = drupalSettings.ephoto_dam.server_url + 'api/' + auth_Id + '/file/getInfo?id=' + id_media;
      var conn = new XMLHttpRequest();
      conn.open("GET", urlInfo);
      conn.send();
      var parser = new DOMParser();

      var self = this;
      conn.onreadystatechange = function () {
        if (conn.readyState === 4) {
          if(conn.status === 200) {

            var xmlDoc = parser.parseFromString(conn.responseText,"text/xml");
            var fields = xmlDoc.getElementById("Desc");

            var newLegend = currentLegend;

            matches.forEach(match => {

              var newValue = null;

              fields.querySelectorAll('field').forEach(element => {
                var fieldName = element.querySelector('name');

                var matchText = match.slice(1,match.length - 1);
                if(fieldName.textContent && fieldName.textContent == matchText ){
                  newValue = element.querySelector('value').textContent;
                  return;
                }
              });

              if(newValue){
                newLegend = newLegend.replaceAll(match,newValue);
              }else{
                newLegend = newLegend.replaceAll(match,"");
              }

            });
            self.setDefaultCaption(id,newLegend);

          }
        }
      };

    }else{
      this.setDefaultCaption(id,currentLegend);
    }
  }

  /**
   * Turns the current string in the editor in a div
   *
   * @returns {DOMParser}
   */
  dataToHtml(){
    var tempdata = this.editor.getData();
    var parser = new DOMParser();
    var doc = parser.parseFromString(tempdata, 'text/html');
    return doc;
  }

  /**
   * Set the dafault caption under the media
   *
   * @param {strign} id id of the media
   * @param {string} caption current caption
   */
  setDefaultCaption(id,caption){
    var doc = this.dataToHtml();
    doc.getElementById("media_" + id).setAttribute("data-caption",caption);
    var retour = doc.querySelector("body");
    this.editor.setData(retour.innerHTML);
  }

  /**
   * Get the image form the embed tag
   *
   * @param {string} str
   * @param {string} filter
   */
  getURLFromString(str,filter) {
    var re = /((([A-Za-z]{3,9}:(?:\/\/)?)(?:[-;:&=+$,\w]+@)?[A-Za-z0-9.-]+|(?:www\.|[-;:&=+$,\w]+@)[A-Za-z0-9.-]+)((?:\/[+~%\/.\w-]*)?\??(?:[-+=&;%@.\w]*)#?\w*)?)/gm;
    var m, result = null;

    while ((m = re.exec(str)) !== null) {
      if (m.index === re.lastIndex) {
        re.lastIndex++;
      }

      if(m[0].indexOf(filter) !== -1) {
        result = m[0];
        break;
      }
    }
    return result;
  }

  /**
   * Returns the value of an xml node
   *
   * @param {tag} xml
   * @param {string} node
   * @returns
   */
  getValueOfXmlNode(xml, node) {
    return xml.getElementsByTagName(node)[0].textContent;
  }

  refresh() {
    const { model } = this.editor;
    const { selection } = model.document;

    // Determine if the cursor (selection) is in a position where adding a
    //  is permitted. This is based on the schema of the model(s)
    // currently containing the cursor.
    const allowedIn = model.schema.findAllowedParent(
      selection.getFirstPosition(),
      'simpleBox',
    );

    // If the cursor is not in a location where it can be added, return
    // null so the addition doesn't happen.
    this.isEnabled = allowedIn !== null;
  }

}
